#!/bin/bash

# Obtiene un cargador/kernel de iPXE según un parámetro dado y lo copia
# al directorio especificado. Utiliza el source oficial y lo compila

### [ ENV ] ###
SCRIPT_DIR="$(dirname "$(readlink -f "${0}")")"
PARENT_DIR="$(dirname "${SCRIPT_DIR}")"
GET_IPXE="${SCRIPT_DIR}/get_ipxe.sh"

### [ DEFAULTS ] ###
IPXE_BIN=${1}
DST_PATH=${2}
SCRIPT_DIR="$(dirname "$(readlink -f "${0}")")"
BUILD_DIR="${SCRIPT_DIR}/ipxe"

### [ FUNCTIONS ] ###
. functions.sh

### [ SCRIPT ] ###
if [ "${#}" -ne 2 ]; then
    echo "Usage: ${0} FILE DEST"
    exit 1
fi

# Make dest dir
mkdir -p ${DST_PATH}

log "Trying to download file from server"
wget "http://boot.ipxe.org/${IPXE_BIN}" -O "${DST_PATH}/${IPXE_BIN}"

# If download failed...
if [ ! -f "${DST_PATH}/${IPXE_BIN}" ]; then
    # Check if source exists and file is build
    if [ ! -f "${BUILD_DIR}/src/bin/${IPXE_BIN}" ]; then
        log "File not found, will be created"
        if [ ! -d "${BUILD_DIR}" ]; then
            log "Downloading repo"
            git clone git://git.ipxe.org/ipxe.git ${BUILD_DIR}
        fi
        
        log "Building required file"
        make -C ${BUILD_DIR}/src bin/${IPXE_BIN}
        if [ $? -ne 0 ]; then
            die "An error has occurred while compiling, aborting. "
        fi
    fi

    if [ -f "${BUILD_DIR}/src/bin/${IPXE_BIN}" ]; then
        log "Copying \"${BUILD_DIR}/src/bin/${IPXE_BIN}\" to \"${DST_PATH}\""
        cp -t ${DST_PATH} ${BUILD_DIR}/src/bin/${IPXE_BIN}
    else
        die "File not found, aborting. "
    fi
fi
