/**
 * ConfigController
 *
 * @description :: Server-side logic for managing configs
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

var fs = require('fs'),
    self = {};

module.exports = _.merge(self, {
    
    /** 
     * Show all config parameters
     */
    niobium: function (req, res, next) {
        res.locals.flash = res.locals.flash || {};
        var alerts = res.locals.flash.alerts = res.locals.flash.alerts || [];
        var cfgValues = _.pick(req.params.all(), _.keys(sails.config.niobium.defaults));
        // sails.log.debug('ConfigValues:', cfgValues);
        
        var selfHarmAlert = {
            context: 'warning',
            title: 'This might break things!',
            message: 'Changing these advanced settings can be harmful to the stability, ' +
                     'security, and performance of this application. You should only continue if ' +
                     'you are sure of what you are doing. —As a well-known web browser said'
        };
        alerts.push(selfHarmAlert);
        
        if ( ! _.isEmpty(cfgValues) ) {
            Config.setParams(cfgValues, true, function (err) {
                if ( err ) {
                    return next(err);
                }
                
                var updatedCfgAlert = {
                    context: 'success',
                    title: 'Updated config',
                    message: 'Successfully updated Niobium config with the submited values.'
                };
                alerts.push(updatedCfgAlert);
                
                return res.view({
                    params: Config.allParams()
                });
            });
        } else {
            return res.view({
                params: Config.allParams()
            });
        }
        
    },
    
    _diff: function (req, res, next, genScript, config, file, success) {
        var updated = req.param('updated');
        
        if ( typeof updated === 'undefined' ) {
            return genScript(function scriptRendered(err, result) {
                if(err) {
                    sails.log.err(err);
                    return next(err);
                }
                
                fs.readFile(file, function (err, data) {
                    if ( err ) {
                        return next(err);
                    }
                    
                    return res.view('config/diff', {
                        config: config,
                        current: String(data).trim(),
                        updated: result.trim()
                    });
                    
                    // sails.log.info('`' + file + '` successfully updated. Please, restart DHCPD Server to apply changes.');
                    // res.type('text/plain');
                    // return res.send(result);
                });
                
            });
            
        } else {
            // Trim and insert bottom new line
            updated = updated.trim() + "\n";
            fs.writeFile(file, updated, function (err) {
                if ( err ) {
                    return next(err);
                }
                
                var fileUpdatedAlert = {
                    title: '`' + file + '` successfully updated',
                    context: 'success',
                    message: success
                };
                
                req.session.flash = {
                    alerts: [fileUpdatedAlert]
                };
                
                return res.redirect(req.originalUrl);
            });
        }
    },
    
    dhcpd: function (req, res, next) {
        var config = 'dhcpd';
        var file = '/srv/nbm/dhcpd.conf';
        var success = 'Please, restart DHCP Server to apply changes: ' +
                      '`systemctl restart dhcpd4` or `service isc-dhcp-server restart`.';
        return self._diff(req, res, next, GenScript.dhcpd, config, file, success);
    },
    
    nfs: function (req, res, next) {
        var config = 'nfs';
        var file = '/srv/nbm/exports';
        var success = 'Please, reload exports with `exportfs -rav` or restart NFS Server to apply changes: ' +
                      '`systemctl restart nfs-config`, `systemctl restart rpc-statd` and `systemctl restart nfs-server`.';
        return self._diff(req, res, next, GenScript.exports, config, file, success);
    }
    
});

