/**
 * ControllerBase
 *
 * @description :: Server-side logic inheritance
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

var that = {};

module.exports = _.merge(that, {
    
    /** 
     * Show all created records.
     */
    index: function (req, res, next) {
        var self = this;
        
        var query = (req.session.management || {})[self._controller()] || {};
        var limit = query.limit || 10;
        var sort = query.sort || {id: 'asc'};
        var page = parseInt(req.param('page'), 10) || 1;
        
        // sails.log.debug('sort', sort);
        
        // Obtenemos una lista de todos los records creados
        self._model().find().paginate({ page: page, limit: limit })
        .sort(sort).exec(function recordsFound(err, records) {
            // Si ha ocurrido algun error, lo devolvemos
            if ( err ) {
                req.session.management = {};
                return next(err);
            }
            
            // Contamos la cantidad de records
            self._model().count(function totalRecords(err, count) {
                // Si ha ocurrido algun error, lo devolvemos
                if ( err ) {
                    return next(err);
                }
                
                // Pasamos el resultado al template
                return res.view('model/list', {
                    model  : { name: self._controller() },
                    records: records,
                    pages  : Math.ceil(count / limit),
                    page   : page,
                    limit  : limit,
                    sort   : sort,
                    fields : self._model().listFields
                });
            });
        });
    },
    
    /** 
     * Destroy a record.
     */
    destroy: function (req, res, next) {
        var self = this;
        var id = req.param('id');
        
        // Buscamos el record dada su ID
        self._model().findOne(id, function recordFound(err, record) {
            // Si ha ocurrido algun error, lo devolvemos
            if ( err ) {
                return next(err);
            }
            
            // Si el record no existe, avisamos del error
            if ( ! record ) {
                var recordNotFoundError = {
                    name: 'recordNotFoundError',
                    message: 'No record found for `' + self._controller() + '` with id `' + id + '`.'
                };
                return next(recordNotFoundError);
            }
            
            // Delete record using the 'destroy' instance method
            record.destroy(function recordDestroyed(err) {
                if ( err ) {
                    return next(err);
                }
                
                // Volvemos a la lista de usuarios
                return res.redirect('/' + self._controller());
            });
        });
    },
    
    /**
     * Destroy multiple records at once.
     */
    destroyMultiple: function (req, res, next) {
        var self = this;
        var ids = req.param('ids') || [];
        ids = Array.isArray(ids) ? ids : [ids];
        
        if ( ! ids.length ) {
            return res.send();
        }
        
        self._model().destroy({id: ids})
        .exec(function(err) {
            if ( err ) {
                return next(err);
            } else {
                // Volvemos a la lista de usuarios
                return res.send('Records destroyed.');
            }
        });
    },
    
    /**
     * Create/Edit Model from 'manage/form.ejs'
     *
     * @param {string} action - Form's action.
     */
    _manageFormAction: function (req, res, next) {
        var self = this;
        var id = req.param('id') || null;
        var obj = _.pick(req.params.all(), self._attributes());
        
        // Check all fields permissions
        var fields = _.cloneDeep(self._model().formFields);
        var objFields = _.keys(obj);
        _.each(fields, function (f) {
            if ( f.perms ) {
                var index = _.indexOf(objFields, f.name);
                if ( index > -1 && ! req.session.User.hasPerm(f.perms, f.anyPerm) ) {
                    delete obj[f.name];
                    sails.log.warn('Trying to edit `' + f.name + '` without permissions.');
                }
            }
        });
        
        // Trim name
        if ( obj.name ) {
            obj.name = obj.name.trim();
        }
        
        // If name declared & empty, error
        if ( typeof obj.name !== 'undefined' && ! obj.name.length ) {
            var emptyNameError = {
                context: 'danger',
                title: 'No name specified',
                message: 'The field `name` cannot be empty.'
            };
            req.session.flash = {
                alerts: [emptyNameError]
            };
        }
        
        var q = id ? self._model().findOne(id) : self._model().findOneByName(obj.name);
        q.exec(function recordFound(err, record) {
            // Si ha ocurrido algun error, lo devolvemos
            if ( err ) {
                return next(err);
            }
            
            // if `id` edit, else create
            if ( id ) {
                // Si el record no existe, avisamos del error
                if ( ! record ) {
                    var recordNotFoundError = {
                        context: 'danger',
                        title: 'Record not found',
                        message: 'No record found for `' + self._controller() + '` with id `' + id + '`.'
                    };
                    req.session.flash = {
                        alerts: [recordNotFoundError]
                    };
                    self._storeObjState(self, obj, req);
                    
                    return res.redirect('/' + self._controller() + '/new');
                    // return next(recordNotFoundError);
                }
                
                // sails.log.debug('obj', obj);
                // collectionsToErase = _.keys(_.filter(obj, function (val) { return Array.isArray(val) && val.length === 0; }));
                // sails.log.debug('collectionsToErase', collectionsToErase);
                
                // Update record
                self._model().update(id, obj, function recordUpdated(err, record) {
                    // Si ha ocurrido algun error volvemos al formulario de edición
                    if ( err ) {
                        sails.log.error(err);
                        var waterlineError = {
                            context: 'danger',
                            title: 'Waterline Error',
                            message: err.toString()
                        };
                        req.session.flash = {
                            alerts: [waterlineError]
                        };
                        self._storeObjState(self, obj, req);
                        
                        return res.redirect('/' + self._controller() + '/edit/' + id);
                    }
                    
                    // Si es actualizado correctamente, lo visualizamos
                    return res.redirect('/' + self._controller() + '/show/' + id);
                });
            } else { // 'create'
                if ( record ) {
                    var nameAlreadyExistsError = {
                        context: 'danger',
                        title: 'Name already exists',
                        message: 'A record with `name`=`' + obj.name + '` already exists.'
                    };
                    req.session.flash = {
                        alerts: [nameAlreadyExistsError]
                    };
                    self._storeObjState(self, obj, req);
                    
                    return res.redirect('/' + self._controller() + '/new');
                    // return next(nameAlreadyExistsError);
                }
                
                // Creamos el usuario
                self._model().create(obj, function recordCreated(err, record) {
                    // Si ocurre algún error, informamos al usuario
                    if ( err ) {
                        sails.log.error(err);
                        var waterlineError = {
                            context: 'danger',
                            title: 'Waterline Error',
                            message: err.toString()
                        };
                        req.session.flash = {
                            alerts: [waterlineError]
                        };
                        self._storeObjState(self, obj, req);
                        
                        // Como ha fallado el formulario, volvemos a él
                        return res.redirect('/' + self._controller() + '/new');
                    }
                    
                    // Si es creado correctamente, lo visualizamos
                    return res.redirect('/' + self._controller() + '/show/' + record.id);
                });
            }
            
        });
    },
    
    /**
     * Store the current object state (race condition!)
     */
    _storeObjState: function (self, obj, req) {
        // sails.log.debug('Storing current state...');
        if ( ! req.session.record ) {
            req.session.record = {};
        }
        req.session.record[self._controller()] = obj;
        // sails.log.debug('req.session.record', obj);
    },
    
    /**
     * Retrieve any previous record state (race condition!)
     */
    _mergeObjState: function (self, obj, req) {
        // sails.log.debug('Merging previous state...');
        // sails.log.debug('pre-obj', obj);
        if ( (req.session || {}).record ) {
            _.merge(obj, req.session.record[self._controller()] || {});
            delete req.session.record[self._controller()];
        }
        // sails.log.debug('post-obj', obj);
        // sails.log.debug('Merged!');
    }
});