/**
 * SessionController
 *
 * @description :: Server-side logic for managing sessions
 * @help        :: See http://links.sailsjs.org/docs/controllers
 */

module.exports = {
    
    /** 
     * Valida el formulario de acceso.
     */
    create: function (req, res, next) {
        var name = (req.param('name') || '').trim();
        var password = req.param('password');
        var url = req.param('url') || '/dashboard';
        
        // Comprobamos que los campos requeridos existan
        if ( ! name || ! password ) {
            var usernamePasswordRequiredError = {
                context: 'danger',
                title: 'Wrong access data',
                message: 'You must enter both, a username and a password.'
            };
            req.session.flash = {
                alerts: [usernamePasswordRequiredError]
            };
            return res.redirect('/');
            // return next(usernamePasswordRequiredError);
        }
        
        // User.findOne({name: {equals: name, caseSensitive: true}})
        User.findOneByName(name)
        .exec(function (err, user) {
            if ( err ) {
                return next(err);
            }
            
            // Si el usuario no existe...
            if ( ! user ) {
                var userNotFoundError = {
                    context: 'danger',
                    title: 'User account not found',
                    message: 'User `' + name + '` not found in database.'
                };
                req.session.flash = {
                    alerts: [userNotFoundError]
                };
                return res.redirect('/');
                // return next(userNotFoundError);
            }
            
            // El usuario existe, comprobamos el password
            if ( ! user.checkPassword(password) ) {
                var usernamePasswordMismatchError = {
                    context: 'danger',
                    title: 'Invalid username or password',
                    message: 'The `username` does not match with given `password`.',
                };
                req.session.flash = {
                    alerts: [usernamePasswordMismatchError],
                    name: name
                };
                return res.redirect('/');
                // return next(usernamePasswordMismatchError);
            }
            
            // Log-in correcto. Almacenamos su ID 
            req.session.userId = user.id;
            // No podemos almacenarlo directamente porque perdemos el método
            // hasPerm. Se ha de cargar en cada consulta.
            // req.session.User = user;
            
            // No comprobamos que tenga el pemiso webAccess, de ello se
            // encargará la Policy definida (para permitir que pueda saltar el
            // aviso mientras está navegando)
            
            // Redirigimos a la vista principal o a la anterior página
            return res.redirect(url);
        });
    },
    
    /** 
     * Elimina una sesión activa
     */
    destroy: function (req, res, next) {
        // Eliminamos las variables de sesión
        req.session.destroy();
        
        // Redirigimos al inicio
        return res.redirect('/');
    }
};
