/**
* Permission.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

var self = {};

module.exports = _.merge(self, {
    
    attributes: {
        
        /**
         * Nombre de la ruta
         */
        name: {
            type: 'string',
            unique: true,
            required: true
        },
        
        /**
         * Build status
         */
        status: {
            type: 'string',
            enum: ['unknown', 'building', 'failed', 'succeeded'],
            defaultsTo: 'unknown'
        },
        
        /**
         * Build log
         */
        log: {
            type: 'text',
            defaultsTo: ''
        },
        
        /**
         * Linked image
         */
        image: {
            model: 'binaryimage',
        }
    },
    
    /**
     * Active builds
     */
    _builds: {},
    
    /**
     * Add new process
     */
    addProcess: function (build, proc) {
        sails.log.debug('Adding build `' + build + '` process...');
        
        if ( typeof self._builds[build] === 'undefined' ) {
            self._builds[build] = proc;
        }
    },
    
    /**
     * Stops process and removes from `_builds`. It is safe to remove an already
     * completed process.
     */
    stopProcess: function (build) {
        var proc = self._builds[build];
        
        sails.log.debug('Stopping build `' + build + '` process...');
        if ( proc && proc.exitCode === null ) {
            proc.stdin.pause();
            proc.kill();
        }
        delete self._builds[build];
    },
    
    // /**
    //  * Form builder
    //  */
    // formFields: [{
    //     name: 'id',
    //     label: 'Id',
    //     type: 'text',
    // }, {
    //     name: 'name',
    //     label: 'Name*',
    //     type: 'text',
    //     attrs: {
    //         'data-writable': true,
    //         'data-bv-stringlength': 'true',
    //         'data-bv-stringlength-min': 3,
    //         'data-bv-notempty': 'true',
    //     }
    // }, {
    //     name: 'roles',
    //     label: 'Roles',
    //     relation: 'role',
    //     type: 'select',
    //     multiple: true,
    //     attrs: {
    //         'data-editable': true,
    //         'title': 'No roles assigned',
    //     },
    //     perms: 'manageRoles',
    //     // Must be injected: { id: '...', name: '...' }
    //     values: []
    // }],
    // 
    // /**
    //  * List Builder. Max col-md-10
    //  */
    // listFields: [{
    //     name: 'id',
    //     label: '#',
    //     'class': 'col-md-1'
    // }, {
    //     name: 'name',
    //     label: 'Name',
    //     'class': 'col-md-9'
    // }]
});

