/**
* Role.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

module.exports = {
    
    attributes: {
        
        /**
         * Nombre del rol.
         */
        name: {
            type: 'string',
            unique: true,
            required: true,
            notNull: true
        },
        
        /**
         * Usuarios con el rol asignado.
         */
        users: {
            collection: 'user',
            via: 'roles'
        },
        
        /**
         * Permisos que otorga el rol.
         */
        permissions: {
            collection: 'permission',
            via: 'roles',
            dominant: true
        },
        
        /**
         * Sobreescribimos la función toJSON.
         */
        toJSON: function () {
            var obj = this.toObject();
            // Evitamos que desaparezcan los permisos al exportar a JSON.
            // FIX: https://github.com/balderdashy/waterline/issues/256#issuecomment-42742446
            if ( (this.permissions || []).length ) {
                obj.permissions = this.permissions.map(function (p) {
                    return ( typeof p.toJSON === 'function' ) ? p.toJSON() : p;
                });
            }
            return obj;
        }
    },
    
    /**
     * Form builder
     */
    formFields: [{
        name: 'id',
        label: 'Id',
        type: 'text',
    }, {
        name: 'name',
        label: 'Name*',
        type: 'text',
        attrs: {
            'data-writable': true,
            'data-bv-stringlength': 'true',
            'data-bv-stringlength-min': 3,
            'data-bv-notempty': 'true',
        }
    }, {
        name: 'permissions',
        label: 'Permissions',
        relation: 'permission',
        type: 'select',
        multiple: true,
        attrs: {
            'data-editable': true,
            'title': 'No permissions assigned',
        },
        perms: 'managePermissions',
        // Must be injected: { id: '...', name: '...' }
        values: []
    }, {
        name: 'users',
        label: 'Users',
        relation: 'user',
        type: 'select',
        multiple: true,
        attrs: {
            'data-editable': true,
            'title': 'No users found.',
        },
        perms: 'manageUsers',
        // Must be injected: { id: '...', name: '...' }
        values: []
    }],
    
    /**
     * List Builder. Max col-md-10
     */
    listFields: [{
        name: 'id',
        label: '#',
        'class': 'col-md-1'
    }, {
        name: 'name',
        label: 'Name',
        'class': 'col-md-9'
    }]
};

