$(document).ready(function () {
    var clients_conn = $('#nbm-connected-clients tbody');
    if ( (clients_conn || []).length ) {
        var clients_act = $('#nbm-clients-activity');
        var clients_act_control = $('#nbm-clients-activity-control');
            
        var now = (new Date()).getTime(),
            active_clients = 0, // clients counter
            refreshInterval = 1, // seconds
            steps = 60, // steps to store
            ticks = 12, // ticks to show
            data = Array.apply(null, Array(steps)).map(function (x, i) { return [now - (steps - i) * 1000, 0]; }); // default data
        
        io.socket.get('/dashboard/subscribe', function (resData) {
            io.socket.on('clientConnected', function (msg) {
                console.log(msg);
                active_clients += 1;
                
                if ( clients_conn.children().length >= 20 ) {
                    clients_conn.children().last().remove();
                }
                
                clients_conn.prepend('<tr>' +
                    '<td>' + msg.id + '</td>' +
                    '<td><a href="/client/show/' + msg.id + '">' + msg.name + '</a></td>' +
                    '<td>' + msg.mac + '</td>' +
                    '<td>' + msg.ip + '</td>' +
                    '<td>' + msg.images + '</td>' +
                    '<td>' + msg.dhcp + '</td>' +
                    '<td>' + msg.timestamp + '</td>' +
                    '</tr>'
                );
            });
        });
        
        var options = {
            xaxis: {
                mode: "time",
                timeformat: "%M:%S",
                tickSize: [steps / ticks, "second" ],
            },
            yaxis: {
                min: 0,
                minTickSize: 1,
                tickDecimals:0
            },
            grid: { hoverable: true },
            series: {
               lines: { show: true, fill: true, },
               points: { show: false },
               shadowSize: 0 // Drawing is faster without shadows
            },
            tooltip: true,
            tooltipOpts: {
                content: "%y client(s) at %x",
                shifts: {
                    x: -60,
                    y: -35
                }
            }
        };
        
        var plot = clients_act.height($('#nbm-management-pages').height())
        .plot([data], options).data('plot');
        
        setInterval(function refreshPlot() {
            var curr_clients = [active_clients, active_clients = 0][0];
            now = (new Date()).getTime();
            
            if ( Math.floor(now / 1000) === Math.floor(data[data.length - 1][0] / 1000) ) {
                // console.log('Sum');
                data[data.length - 1][1] += curr_clients;
            } else {
                // console.log('Append');
                data.push([now, curr_clients]);
                if ( data.length > steps) {
                    data.shift();
                }
            }
            
            if ( clients_act_control.data('state') === 'playing' ) {
                // plot.setData([data]);
                // plot.setupGrid();
                // plot.draw();
                
                var lastElem = data.length - 1;
                var lastVal = data[lastElem][1];
                var tmpData = $.extend(true, [], data);
                
                plot.getOptions().xaxes[0].max = now;
                
                clients_act.animate( { 1:1 }, {
                    duration: 50,
                    step: function ( now, fx ) {
                        tmpData[lastElem][1] = lastVal * fx.pos;
                        // console.log(tmpData[lastElem][1]);
                        plot.setData([tmpData]);
                        plot.setupGrid();
                        plot.draw();
                    }
                });
                
            }
        }, refreshInterval * 1000);
        clients_act_control.off('click').on('click', function (e) {
            e.preventDefault();
            e.stopPropagation();
            
            var button = $(this);
            var icon = button.find('.glyphicon');
            if ( button.data('state') === 'playing' ) {
                button.data('state', 'stopped');
                icon.removeClass('glyphicon-pause').addClass('glyphicon-play');
            } else {
                button.data('state', 'playing');
                icon.addClass('glyphicon-pause').removeClass('glyphicon-play');
            }
            
            return false;
        });
    }
    
});
