$(document).ready(function () {
    // Enable Bootstrap's tooltip in each button
    $('button[data-toggle="tooltip"]').tooltip();
    
    // Enable Bootstrap's selectpicker in each select
    $('.selectpicker').selectpicker();
    
    $('.nbm-model-list[data-model]').each(function () {
        var list = $(this);
        var globalButtons = list.find('.nbm-model-list-global [data-action]');
        var localActions = list.find('.nbm-model-list-local[data-id]');
        var paginateAnchors = list.find('.nbm-model-list-pagination [data-page]');
        
        // Security check
        if ( list.find(list).length ) {
            console.error('Nested Model lists are not supported!');
            return false;
        }
        
        // Model data
        var modelName = list.data('model');
        
        if ( ! modelName ) {
            console.error('Wrong model data!');
            return false;
        }
        
        // Local actions
        localActions.each(function () {
            var action = $(this);
            var localButtons = action.find('[data-action]').add(action.filter('[data-action]'));
            
            localButtons.off('click').on('click', function (event) {
                // Disable propagation & default action 
                event.stopPropagation();
                event.preventDefault();
                // Store element
                var elem = $(this);
                // Model data
                var modelId = action.data('id');
                var modelAction = elem.data('action');
                
                if ( ! modelId || ! modelAction ) {
                    console.error('Wrong model data!');
                    return false;
                }
                
                switch ( modelAction ) {
                    case 'select':
                        elem.toggleClass('info', !elem.hasClass('info'));
                        break;
                    default:
                        if ( modelAction !== 'destroy' || confirm('Are you sure?') ) {
                            window.location.href = '/' + modelName + '/' + modelAction + '/' + modelId;
                        }
                }
            });
        });
        
        // Global buttons
        globalButtons.off('click').on('click', function (event) {
            // Disable propagation & default action
            // event.stopPropagation();
            event.preventDefault();
            // Store button
            var button = $(this);
            
            switch ( button.data('action') ) {
                case 'create':
                    window.location.href = '/' + modelName + '/new';
                    break;
                case 'bulkaction':
                    var action = button.data('bulkaction');
                    var arr = $.map(localActions.filter('.info[data-action="select"]'), function (n) {
                        return $(n).data('id');
                    });
                    if ( action && arr.length && confirm('Are you sure?')) {
                        console.log(action + ':', arr);
                        $.ajax({
                          type: 'POST',
                          url: '/' + modelName + '/destroyMultiple',
                          data: { ids: arr }
                        }).done(function( msg ) {
                            alert('Done: ' + msg);
                            window.location.href = '/' + modelName;
                        }).fail(function( jqXHR, textStatus ) {
                            alert('Fail: ' + textStatus);
                            window.location.href = '/' + modelName;
                        });
                    }
                    break;
                case 'limit':
                    var limit = button.data('limit');
                    io.socket.get('/limit/' + modelName + '/' + limit, function (resData) {
                        window.location.replace('/' + modelName);
                    });
                    break;
                case 'sort':
                    var col = button.data('sort-col');
                    var dir = button.data('sort-dir');
                    io.socket.get('/sort/' + modelName + '/' + col + '/' + dir, function (resData) {
                        window.location.replace('/' + modelName);
                    });
                    break;
            }
        });
        
        // Set paginators
        paginateAnchors.off('click').on('click', function (event) {
            // Disable propagation & default action
            event.stopPropagation();
            event.preventDefault();
            // Store button
            var anchor = $(this);
            if ( ! anchor.parent().is('.active, .disabled') ) {
                window.location.href = '/' + modelName + '/list/' + anchor.data('page');
            }
            return false;
        });
    });
    
    $('.nbm-model-form[data-model][data-id]').each(function () {
        // Get form and initialize Bootstrap Validator
        var form = $(this).bootstrapValidator();
        var bv = form.data('bootstrapValidator');
        
        // Security check
        if ( form.find(form).length ) {
            console.error('Nested Model forms are not supported!');
            return false;
        }
        
        // Model data
        var modelName = form.data('model');
        var modelId = form.data('id');
        
        if ( ! modelName || ! modelId ) {
            console.error('Wrong model data!');
            return false;
        }
        
        // Form elements
        var inputs = form.find('input, select, textarea')
        .not('.bootstrap-select-searchbox input, [type="hidden"]')
        .each(function () {
            // Store all default values
            var input = $(this);
            input.data('default', input.val());
        });
        var buttons = form.find('[data-action]');
        
        // Change form action on event trigger
        form.off('nbm-model-create nbm-model-update nbm-model-show') // Remove prev events
        .on('nbm-model-create nbm-model-update nbm-model-show', function (event) {
            inputs.each(function () {
                var input = $(this);
                input.prop('disabled', true).val(input.data('default'));
            });
            inputs.filter('select').find('option[data-selected]').attr('selected', true);
            inputs.filter('[type="checkbox"]').each(function () {
                var chk = $(this);
                chk.prop('checked', !!chk.is('[data-checked]'));
            });
            buttons.not('[data-action="list"], [data-action="relation"]').addClass('hidden');
        }).on('nbm-model-create', function (event) {
            form.data('action', 'create').attr('action', '/' + modelName + '/create');
            inputs.filter('[data-writable], [data-editable]').prop('disabled', false).first().focus();
            inputs.filter('.selectpicker').selectpicker('refresh');
            buttons.filter('[data-action="submit"]').removeClass('hidden');
        }).on('nbm-model-update', function (event) {
            form.data('action', 'update').attr('action', '/' + modelName + '/update/' + modelId);
            inputs.filter('[data-editable]').prop('disabled', false).first().focus();
            inputs.filter('.selectpicker').selectpicker('refresh');
            buttons.filter('[data-action="show"], [data-action="submit"]').removeClass('hidden');
        }).on('nbm-model-show', function (event) {
            bv.resetForm();
            form.data('action', 'show').attr('action', '');
            inputs.filter('.selectpicker').selectpicker('refresh');
            buttons.filter('[data-action="update"]').removeClass('hidden');
        });
        
        // Set buttons' actions
        buttons.off('click').on('click', function (event) {
            // Disable propagation & default action
            //event.stopPropagation();
            event.preventDefault();
            
            var button = $(this);
            
            switch ( button.data('action') ) {
                case 'update':
                    form.trigger('nbm-model-update');
                    break;
                case 'show':
                    if ( ! window.onbeforeunload || confirm('All unsaved changes will be lost, continue?') ) {
                        window.onbeforeunload = null;
                        form.trigger('nbm-model-show');
                    }
                    break;
                case 'submit':
                    window.onbeforeunload = null;
                    form.trigger('submit');
                    break;
                case 'list':
                    window.location.href = '/' + modelName;
                    break;
                case 'relation':
                    var targetId = $(button.data('target')).val();
                    if ( Array.isArray(targetId) ) {
                        targetId = ( targetId.length === 1  ) ? targetId[0] : null;
                    }
                    if ( targetId ) {
                        window.location.href = '/' + button.data('model') + '/show/' + targetId;
                    } else {
                        window.location.href = '/' + button.data('model');
                    }
                    break;
            }
        });
        
        // create, show, update
        form.trigger('nbm-model-' + form.data('action'));
        
        // Interrupt navigation if any change is made
        inputs.on('change', function(e) {
            if ( ! window.onbeforeunload ) {
                window.onbeforeunload = function() {
                    return 'All unsaved changes will be lost, continue?';
                };
            }
        });
        
        // Custom Bootstrap Validator fixes
        
        /**
         * Allows form to send zero selected elements. Fixes nullable collections
         */
        var relsToFix = function (fields) {
            $.each(fields, function (i, name) {
                name += '[]';
                var input = form.find('select[name="' + name + '"]');
                // If element exists and has no value assigned
                if ( (input || []).length === 1 && ! input.val() ) {
                    // Replace with hidden input
                    input.replaceWith($('<input />').attr('name', name).attr('type', 'hidden').val(''));
                }
            });
        };
        
        /**
         * Prevents field submit if not changed
         */
        var fieldsToStop = function (fields) {
            $.each(fields, function (i, name) {
                var input = form.find('[name="' + name + '"]');
                // If element exists and has `data-default`
                // if ( (input || []).length === 1 && input.is('[data-default]') ) {
                if ( (input || []).length === 1 && typeof input.data('default') !== 'undefined' ) {
                    // Check if value has changed
                    if ( input.val().trim() === (input.data('default') || '').trim() ) {
                        input.prop('disabled', true);
                    }
                }
            });
        };
        
        // Form submitted as POST, change to sockets!!
        form.attr('method', 'post').on('success.form.bv', function(e) {
            if ( ['create', 'update'].indexOf(form.data('action')) > -1 ) {
                switch ( modelName ) {
                    case 'binaryimage':
                        relsToFix(['groups']);
                        break;
                    case 'client':
                        relsToFix(['groups']);
                        break;
                    case 'clientgroup':
                        relsToFix(['clients', 'images']);
                        break;
                    // case 'connector':
                    //     break;
                    case 'permission':
                        relsToFix(['roles']);
                        break;
                    case 'role':
                        relsToFix(['users', 'permissions']);
                        break;
                    case 'user':
                        relsToFix(['roles']);
                        break;
                }
                
                // Avoid sending an existing name (waterline bug)
                if ( form.data('action') === 'update' ) {
                    fieldsToStop(['name']);
                }
                
                form[0].submit();
            }
        });
    });
    
});
