/**
 * Route Mappings
 * (sails.config.routes)
 *
 * Your routes map URLs to views and controllers.
 *
 * If Sails receives a URL that doesn't match any of the routes below,
 * it will check for matching files (images, scripts, stylesheets, etc.)
 * in your assets directory.  e.g. `http://localhost:1337/images/foo.jpg`
 * might match an image file: `/assets/images/foo.jpg`
 *
 * Finally, if those don't match either, the default 404 handler is triggered.
 * See `config/404.js` to adjust your app's 404 logic.
 *
 * Note: Sails doesn't ACTUALLY serve stuff from `assets`-- the default Gruntfile in Sails copies
 * flat files from `assets` to `.tmp/public`.  This allows you to do things like compile LESS or
 * CoffeeScript for the front-end.
 *
 * For more information on routes, check out:
 * http://links.sailsjs.org/docs/config/routes
 */

module.exports.routes = {

    // Make the view located at `views/homepage.ejs` (or `views/homepage.jade`, etc. depending on your
    // default view engine) your home page.
    //
    // (Alternatively, remove this and add an `index.html` file in your `assets` directory)
    '/': {
        controller: 'index'
    },
    
    '/limit/:model/:limit': {
        controller: 'index',
        action    : 'limit'
    },
    '/sort/:model/:col/:dir': {
        controller: 'index',
        action    : 'sort'
    },
    
    // models/list routes
    '/user/list/:page?': {
        controller: 'user',
        action    : 'index'
    },
    '/role/list/:page?': {
        controller: 'role',
        action    : 'index'
    },
    '/permission/list/:page?': {
        controller: 'permission',
        action    : 'index'
    },
    '/binaryimage/list/:page?': {
        controller: 'binaryimage',
        action    : 'index'
    },
    '/client/list/:page?': {
        controller: 'client',
        action    : 'index'
    },
    '/clientgroup/list/:page?': {
        controller: 'clientgroup',
        action    : 'index'
    },
    
    // iPXE script route
    '/boot': {
        controller: 'core',
        action    : 'boot'
    },
    '/getscript/:mac': {
        controller: 'core',
        action    : 'getscript'
    },
    '/getfile/:id/:file': {
        controller: 'core',
        action    : 'getfile'
    },
    '/core/genbuild/:connector': {
        controller: 'core',
        action    : 'genbuild'
    },
    
    '/build/start': 'Build.start',
    '/build/show/:id': 'Build.show',
    '/build/stop': 'Build.stop',
    
    '/config/niobium': 'Config.niobium',
    '/config/dhcpd': 'Config.dhcpd',
    '/config/nfs': 'Config.nfs',
    
    // Controllers' routes (FIX! [Backbone + REST?])
    // 'GET /binaryimage/': 'BinaryImage.index',
    // 'GET /binaryimage/index': 'BinaryImage.index',
    // 'GET /binaryimage/new': 'BinaryImage.new',
    // 'GET /binaryimage/create': 'BinaryImage.create',
    // 'GET /binaryimage/show/:id': 'BinaryImage.show',
    // 'GET /binaryimage/edit/:id': 'BinaryImage.edit',
    // 'GET /binaryimage/update/:id': 'BinaryImage.update',
    // 'GET /binaryimage/destroy/:id': 'BinaryImage.destroy',
    // 'GET /binaryimage/destroymultiple': 'BinaryImage.destroyMultiple',
    
    // 'get /user/show/:id': {
    //     controller: 'user',
    //     action    : 'show'
    // }


    // If a request to a URL doesn't match any of the custom routes above,
    // it is matched against Sails route blueprints.  See `config/blueprints.js`
    // for configuration options and examples.

};
